import * as React from "react";
import { SealedInitialState } from "reakit-utils/useSealedState";
import { DisclosureState, DisclosureActions, DisclosureInitialState, DisclosureStateReturn } from "../Disclosure/DisclosureState";
export declare type DialogState = DisclosureState & {
    /**
     * Toggles Dialog's `modal` state.
     *   - Non-modal: `preventBodyScroll` doesn't work and focus is free.
     *   - Modal: `preventBodyScroll` is automatically enabled, focus is
     * trapped within the dialog and the dialog is rendered within a `Portal`
     * by default.
     */
    modal: boolean;
};
export declare type DialogActions = DisclosureActions & {
    /**
     * Sets `modal`.
     */
    setModal: React.Dispatch<React.SetStateAction<DialogState["modal"]>>;
};
export declare type DialogInitialState = DisclosureInitialState & Partial<Pick<DialogState, "modal">>;
export declare type DialogStateReturn = DisclosureStateReturn & DialogState & DialogActions & {
    /**
     * @private
     */
    unstable_modal: boolean;
};
export declare function useDialogState(initialState?: SealedInitialState<DialogInitialState>): DialogStateReturn;
export declare namespace useDialogState {
    var __keys: ("visible" | "hide" | "show" | "baseId" | "unstable_idCountRef" | "unstable_setBaseId" | "unstable_animated" | "unstable_animating" | "toggle" | "unstable_stopAnimation" | "unstable_setIsMounted" | "modal" | "setModal" | "unstable_modal")[];
}
